/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/


#ifndef __CREATE_DATABASE_H__
#define __CREATE_DATABASE_H__

#define CREATE_DB_V1 "\
BEGIN TRANSACTION;\
CREATE TABLE \"tasks\" (\
\"task_id\" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL DEFAULT (1),\
\"status\" INTEGER,\
\"queued\" DATE,\
\"started\" DATE,\
\"finished\" DATE,\
\"type\" INTEGER,\
\"parameters\" TEXT DEFAULT NULL\
);\
CREATE TABLE \"taskdata_test_transmission_quality\" (\
\"dsid\" CHAR(24),\
\"upstream\" INTEGER,\
\"downstream\" INTEGER,\
\"timestamp\" DATE,\
\"errorcode\" INTEGER,\
\"task_id\" INTEGER,\
FOREIGN KEY(\"task_id\") REFERENCES tasks(task_id)\
);\
CREATE TABLE \"dsa_config\" (\
\"key\" varchar(40) primary key NOT NULL,\
\"value\" varchar(255) NOT NULL\
);\
CREATE INDEX dsa_config_key ON dsa_config(key);\
INSERT INTO \"dsa_config\" VALUES('db_version', '1');\
CREATE TRIGGER \"transmission_quality_taskdata_cleanup\" \
BEFORE DELETE ON \"tasks\" \
FOR EACH ROW BEGIN \
DELETE FROM \"taskdata_test_transmission_quality\" WHERE \"task_id\" = OLD.task_id;\
END;\
COMMIT;"

#define CREATE_DB_V2 "\
BEGIN TRANSACTION;\
CREATE TABLE \"taskdata_test_device_presence\" (\
\"dsid\" CHAR(24),\
\"dsmdsid\" CHAR(24),\
\"deviceid\" INTEGER,\
\"resultcode\" INTEGER,\
\"timestamp\" DATE,\
\"task_id\" INTEGER,\
FOREIGN KEY(\"task_id\") REFERENCES tasks(task_id)\
);\
CREATE TABLE \"taskdata_device_statistic\" (\
\"dsid\" CHAR(24),\
\"dsmdsid\" CHAR(24),\
\"deviceid\" INTEGER,\
\"value\" INTEGER,\
\"timestamp\" DATE,\
\"errorcode\" INTEGER,\
\"stat_id\" INTEGER,\
\"task_id\" INTEGER,\
FOREIGN KEY(\"stat_id\") REFERENCES dsa_statistic(stat_id)\
FOREIGN KEY(\"task_id\") REFERENCES tasks(task_id)\
);\
CREATE TABLE \"taskdata_meter_statistic\" (\
\"dsmdsid\" CHAR(24),\
\"value\" INTEGER,\
\"timestamp\" DATE,\
\"errorcode\" INTEGER,\
\"stat_id\" INTEGER,\
\"task_id\" INTEGER,\
FOREIGN KEY(\"stat_id\") REFERENCES dsa_statistic(stat_id)\
FOREIGN KEY(\"task_id\") REFERENCES tasks(task_id)\
);\
CREATE TABLE \"taskdata_upstream_metric\" (\
\"dsid\" CHAR(24),\
\"dsmdsid\" CHAR(24),\
\"deviceid\" INTEGER,\
\"quality\" INTEGER,\
\"polarity\" INTEGER,\
\"crosstalk\" INTEGER,\
\"packettype\" INTEGER,\
\"timestamp\" DATE\
);\
ALTER TABLE \"tasks\" ADD \"parent_task_id\" INTEGER;\
CREATE TABLE \"dsms\" (\
\"dsid\" CHAR(24) PRIMARY KEY NOT NULL,\
\"hwversion\" INTEGER,\
\"armswversion\" INTEGER,\
\"dspswversion\" INTEGER,\
\"apiversion\" INTEGER,\
\"name\" CHAR(21),\
\"present\" INTEGER,\
\"apartmentid\" INTEGER,\
\"configid\" INTEGER,\
\"countdevices\" INTEGER,\
\"countactivedevices\" INTEGER,\
\"countinactivedevices\" INTEGER\
);\
CREATE TABLE \"dsm_zones\" (\
\"dsid\" CHAR(24),\
\"index\" INTEGER,\
\"zoneid\" INTEGER,\
FOREIGN KEY(\"dsid\") REFERENCES dsms(dsid)\
);\
CREATE TRIGGER \"dsm_zonedata_cleanup\" \
BEFORE DELETE ON \"dsms\" \
FOR EACH ROW BEGIN \
DELETE FROM \"dsm_zones\" WHERE \"dsid\" = OLD.dsid;\
END;\
CREATE TABLE \"devices\" (\
\"key\" CHAR(54) PRIMARY KEY NOT NULL,\
\"dsid\" CHAR(24),\
\"dsmdsid\" CHAR(24),\
\"deviceid\" INTEGER,\
\"productid\" INTEGER,\
\"functionid\" INTEGER,\
\"version\" INTEGER,\
\"zoneid\" INTEGER,\
\"state\" INTEGER,\
\"locked\" INTEGER,\
\"outputmode\" INTEGER,\
\"name\" CHAR(21),\
\"serialnumber\" INTEGER\
);\
UPDATE \"dsa_config\" SET value=\'2\' WHERE key=\'db_version\';\
INSERT INTO \"dsa_config\" VALUES('MeterPingMode', '3');\
INSERT INTO \"dsa_config\" VALUES('MeterTemperatureWarning', '90');\
INSERT INTO \"dsa_config\" VALUES('MeterTemperatureError', '150');\
COMMIT;\
"
#define CREATE_DB_V3 "\
BEGIN TRANSACTION;\
ALTER TABLE \"devices\" ADD \"ltmode\" INTEGER;\
UPDATE \"dsa_config\" SET value=\'3\' WHERE key=\'db_version\';\
COMMIT;\
"
#define CREATE_DB_V4 "\
BEGIN TRANSACTION;\
CREATE TABLE \"dsm_state\" (\
\"dsid\" CHAR(24) PRIMARY KEY NOT NULL,\
\"hardware\" INTEGER,\
\"communicationdsp\" INTEGER,\
\"temperature\" INTEGER,\
\"configuration\" INTEGER,\
\"ds485bus\" INTEGER,\
\"powerline\" INTEGER,\
FOREIGN KEY(\"dsid\") REFERENCES dsms(dsid)\
);\
UPDATE \"dsa_config\" SET value=\'4\' WHERE key=\'db_version\';\
COMMIT;\
"
#define CREATE_DB_V5 "\
BEGIN TRANSACTION;\
ALTER TABLE \"devices\" ADD \"configurationid\" INTEGER;\
ALTER TABLE \"tasks\" ADD \"persistent\" INTEGER;\
UPDATE \"dsa_config\" SET value=\'5\' WHERE key=\'db_version\';\
COMMIT;\
"
#define CREATE_DB_V6 "\
BEGIN TRANSACTION;\
CREATE TABLE \"taskdata_device_config_update\" (\
\"key\" CHAR(54) PRIMARY KEY NOT NULL,\
\"status\" INTEGER,\
\"errorcode\" INTEGER,\
\"timestamp\" DATE,\
FOREIGN KEY(\"key\") REFERENCES devices(key)\
);\
CREATE TRIGGER \"taskdata_device_config_update_cleanup\" \
BEFORE DELETE ON \"devices\" \
FOR EACH ROW BEGIN \
DELETE FROM \"taskdata_device_config_update\" WHERE \"key\" = OLD.key;\
END;\
UPDATE \"dsa_config\" SET value=\'6\' WHERE key=\'db_version\';\
COMMIT;\
"

#define CREATE_DB_V7 "\
BEGIN TRANSACTION;\
ALTER TABLE \"devices\" ADD \"deleteflag\" INTEGER;\
CREATE TRIGGER \"dsm_cleanup\" \
BEFORE DELETE ON \"dsms\" \
FOR EACH ROW BEGIN \
DELETE FROM \"devices\" WHERE \"dsmdsid\" = OLD.dsid;\
DELETE FROM \"dsm_state\" WHERE \"dsid\" = OLD.dsid;\
END;\
UPDATE \"dsa_config\" SET value=\'7\' WHERE key=\'db_version\';\
COMMIT;\
"

/* migration of dsid to dsuid */
#define CREATE_DB_V8 "\
BEGIN TRANSACTION;\
\
ALTER TABLE \"taskdata_test_transmission_quality\" RENAME TO \"tmp_taskdata_test_transmission_quality\";\
CREATE TABLE \"taskdata_test_transmission_quality\" (\
\"dsuid\" CHAR(35),\
\"upstream\" INTEGER,\
\"downstream\" INTEGER,\
\"timestamp\" DATE,\
\"errorcode\" INTEGER,\
\"task_id\" INTEGER,\
FOREIGN KEY(\"task_id\") REFERENCES tasks(task_id)\
);\
INSERT INTO \"taskdata_test_transmission_quality\" (\"dsuid\",\"upstream\",\"downstream\",\"timestamp\",\"errorcode\",\"task_id\")\
SELECT \"dsid\",\"upstream\",\"downstream\",\"timestamp\",\"errorcode\",\"task_id\" FROM \"tmp_taskdata_test_transmission_quality\";\
DROP TABLE \"tmp_taskdata_test_transmission_quality\";\
\
ALTER TABLE \"taskdata_test_device_presence\" RENAME TO \"tmp_taskdata_test_device_presence\";\
CREATE TABLE \"taskdata_test_device_presence\" (\
\"dsuid\" CHAR(35),\
\"dsmdsuid\" CHAR(35),\
\"deviceid\" INTEGER,\
\"resultcode\" INTEGER,\
\"timestamp\" DATE,\
\"task_id\" INTEGER,\
FOREIGN KEY(\"task_id\") REFERENCES tasks(task_id)\
);\
INSERT INTO \"taskdata_test_device_presence\" (\"dsuid\",\"dsmdsuid\",\"deviceid\",\"resultcode\",\"timestamp\",\"task_id\") \
SELECT \"dsid\",\"dsmdsid\",\"deviceid\",\"resultcode\",\"timestamp\",\"task_id\" FROM \"tmp_taskdata_test_device_presence\";\
DROP TABLE \"tmp_taskdata_test_device_presence\";\
\
ALTER TABLE \"taskdata_device_statistic\" RENAME TO \"tmp_taskdata_device_statistic\";\
CREATE TABLE \"taskdata_device_statistic\" (\
\"dsuid\" CHAR(35),\
\"dsmdsuid\" CHAR(35),\
\"deviceid\" INTEGER,\
\"value\" INTEGER,\
\"timestamp\" DATE,\
\"errorcode\" INTEGER,\
\"stat_id\" INTEGER,\
\"task_id\" INTEGER,\
FOREIGN KEY(\"stat_id\") REFERENCES dsa_statistic(stat_id)\
FOREIGN KEY(\"task_id\") REFERENCES tasks(task_id)\
);\
INSERT INTO \"taskdata_device_statistic\" (\"dsuid\",\"dsmdsuid\",\"deviceid\",\"value\",\"timestamp\",\"errorcode\",\"stat_id\",\"task_id\") \
SELECT \"dsid\",\"dsmdsid\",\"deviceid\",\"value\",\"timestamp\",\"errorcode\",\"stat_id\",\"task_id\" FROM \"tmp_taskdata_device_statistic\";\
DROP TABLE \"tmp_taskdata_device_statistic\";\
\
ALTER TABLE \"taskdata_meter_statistic\" RENAME TO \"tmp_taskdata_meter_statistic\";\
CREATE TABLE \"taskdata_meter_statistic\" (\
\"dsmdsuid\" CHAR(35),\
\"value\" INTEGER,\
\"timestamp\" DATE,\
\"errorcode\" INTEGER,\
\"stat_id\" INTEGER,\
\"task_id\" INTEGER,\
FOREIGN KEY(\"stat_id\") REFERENCES dsa_statistic(stat_id)\
FOREIGN KEY(\"task_id\") REFERENCES tasks(task_id)\
);\
INSERT INTO \"taskdata_meter_statistic\" (\"dsmdsuid\",\"value\",\"timestamp\",\"errorcode\",\"stat_id\",\"task_id\") \
SELECT \"dsmdsid\",\"value\",\"timestamp\",\"errorcode\",\"stat_id\",\"task_id\" FROM \"tmp_taskdata_meter_statistic\";\
DROP TABLE \"tmp_taskdata_meter_statistic\";\
\
ALTER TABLE \"taskdata_upstream_metric\" RENAME TO \"tmp_taskdata_upstream_metric\";\
CREATE TABLE \"taskdata_upstream_metric\" (\
\"dsuid\" CHAR(35),\
\"dsmdsuid\" CHAR(35),\
\"deviceid\" INTEGER,\
\"quality\" INTEGER,\
\"polarity\" INTEGER,\
\"crosstalk\" INTEGER,\
\"packettype\" INTEGER,\
\"timestamp\" DATE\
);\
INSERT INTO \"taskdata_upstream_metric\" (\"dsuid\",\"dsmdsuid\",\"deviceid\",\"quality\",\"polarity\",\"crosstalk\",\"packettype\",\"timestamp\") \
SELECT \"dsid\",\"dsmdsid\",\"deviceid\",\"quality\",\"polarity\",\"crosstalk\",\"packettype\",\"timestamp\" FROM \"tmp_taskdata_upstream_metric\";\
DROP TABLE \"tmp_taskdata_upstream_metric\";\
\
ALTER TABLE \"dsms\" RENAME TO \"tmp_dsms\";\
CREATE TABLE \"dsms\" (\
\"dsuid\" CHAR(35) PRIMARY KEY NOT NULL,\
\"hwversion\" INTEGER,\
\"armswversion\" INTEGER,\
\"dspswversion\" INTEGER,\
\"apiversion\" INTEGER,\
\"name\" CHAR(21),\
\"present\" INTEGER,\
\"apartmentid\" INTEGER,\
\"configid\" INTEGER,\
\"countdevices\" INTEGER,\
\"countactivedevices\" INTEGER,\
\"countinactivedevices\" INTEGER\
);\
INSERT INTO \"dsms\" (\"dsuid\",\"hwversion\",\"armswversion\",\"dspswversion\",\"apiversion\",\"name\",\"present\",\"apartmentid\",\"configid\",\"countdevices\",\"countactivedevices\",\"countinactivedevices\") \
SELECT \"dsid\",\"hwversion\",\"armswversion\",\"dspswversion\",\"apiversion\",\"name\",\"present\",\"apartmentid\",\"configid\",\"countdevices\",\"countactivedevices\",\"countinactivedevices\" FROM \"tmp_dsms\";\
DROP TABLE \"tmp_dsms\";\
\
ALTER TABLE \"dsm_zones\" RENAME TO \"tmp_dsm_zones\";\
CREATE TABLE \"dsm_zones\" (\
\"dsuid\" CHAR(35),\
\"index\" INTEGER,\
\"zoneid\" INTEGER,\
FOREIGN KEY(\"dsuid\") REFERENCES dsms(dsuid)\
);\
INSERT INTO \"dsm_zones\" (\"dsuid\",\"index\",\"zoneid\") \
SELECT \"dsid\",\"index\",\"zoneid\" FROM \"tmp_dsm_zones\";\
DROP TABLE \"tmp_dsm_zones\";\
\
DROP TRIGGER IF EXISTS \"dsm_zonedata_cleanup\";\
CREATE TRIGGER \"dsm_zonedata_cleanup\" \
BEFORE DELETE ON \"dsms\" \
FOR EACH ROW BEGIN \
DELETE FROM \"dsm_zones\" WHERE \"dsuid\" = OLD.dsuid;\
END;\
\
ALTER TABLE \"devices\" RENAME TO \"tmp_devices\";\
CREATE TABLE \"devices\" (\
\"key\" CHAR(54) PRIMARY KEY NOT NULL,\
\"dsuid\" CHAR(35),\
\"dsmdsuid\" CHAR(35),\
\"deviceid\" INTEGER,\
\"productid\" INTEGER,\
\"functionid\" INTEGER,\
\"version\" INTEGER,\
\"zoneid\" INTEGER,\
\"state\" INTEGER,\
\"locked\" INTEGER,\
\"outputmode\" INTEGER,\
\"name\" CHAR(21),\
\"serialnumber\" INTEGER,\
\"ltmode\" INTEGER,\
\"configurationid\" INTEGER,\
\"deleteflag\" INTEGER\
);\
INSERT INTO \"devices\" (\"key\",\"dsuid\",\"dsmdsuid\",\"deviceid\",\"productid\",\"functionid\",\"version\",\"zoneid\",\"state\",\"locked\",\"outputmode\",\"name\",\"serialnumber\",\"ltmode\",\"configurationid\",\"deleteflag\") \
SELECT \"key\",\"dsid\",\"dsmdsid\",\"deviceid\",\"productid\",\"functionid\",\"version\",\"zoneid\",\"state\",\"locked\",\"outputmode\",\"name\",\"serialnumber\",\"ltmode\",\"configurationid\",\"deleteflag\" FROM \"tmp_devices\";\
DROP TABLE \"tmp_devices\";\
\
ALTER TABLE \"dsm_state\" RENAME TO \"tmp_dsm_state\";\
CREATE TABLE \"dsm_state\" (\
\"dsuid\" CHAR(35) PRIMARY KEY NOT NULL,\
\"hardware\" INTEGER,\
\"communicationdsp\" INTEGER,\
\"temperature\" INTEGER,\
\"configuration\" INTEGER,\
\"ds485bus\" INTEGER,\
\"powerline\" INTEGER,\
FOREIGN KEY(\"dsuid\") REFERENCES dsms(dsuid)\
);\
INSERT INTO \"dsm_state\" (\"dsuid\",\"hardware\",\"communicationdsp\",\"temperature\",\"configuration\",\"ds485bus\",\"powerline\") \
SELECT \"dsid\",\"hardware\",\"communicationdsp\",\"temperature\",\"configuration\",\"ds485bus\",\"powerline\" FROM \"tmp_dsm_state\";\
DROP TABLE \"tmp_dsm_state\";\
\
DROP TRIGGER IF EXISTS \"dsm_cleanup\";\
CREATE TRIGGER \"dsm_cleanup\" \
BEFORE DELETE ON \"dsms\" \
FOR EACH ROW BEGIN \
DELETE FROM \"devices\" WHERE \"dsmdsuid\" = OLD.dsuid;\
DELETE FROM \"dsm_state\" WHERE \"dsuid\" = OLD.dsuid;\
END;\
\
UPDATE \"dsa_config\" SET value=\'8\' WHERE key=\'db_version\';\
COMMIT;\
"

/* introduce virtual devices */
#define CREATE_DB_V9 "\
BEGIN TRANSACTION;\
ALTER TABLE \"dsms\" ADD \"virtual\" INTEGER;\
ALTER TABLE \"devices\" ADD \"virtual\" INTEGER;\
ALTER TABLE \"devices\" ADD \"hardwareguid\" VARCHAR(40);\
UPDATE \"dsa_config\" SET value=\'9\' WHERE key=\'db_version\';\
COMMIT;\
"

/* introduce  hardwareinfo for virtual devices */
#define CREATE_DB_V10 "\
BEGIN TRANSACTION;\
ALTER TABLE \"devices\" ADD \"hardwareinfo\" VARCHAR(40);\
UPDATE \"dsa_config\" SET value=\'10\' WHERE key=\'db_version\';\
COMMIT;\
"

/* remove column countactivedevices and countinactivedevices from dsms
 * drop table dsm_zones (unused)
 * add triggers to update device count when inserting/deleting devices */
#define CREATE_DB_V11 "\
BEGIN TRANSACTION;\
ALTER TABLE \"dsms\" RENAME TO \"tmp_dsms\";\
CREATE TABLE \"dsms\" (\
\"dsuid\" CHAR(35) PRIMARY KEY NOT NULL,\
\"hwversion\" INTEGER,\
\"armswversion\" INTEGER,\
\"dspswversion\" INTEGER,\
\"apiversion\" INTEGER,\
\"name\" CHAR(21),\
\"present\" INTEGER,\
\"apartmentid\" INTEGER,\
\"configid\" INTEGER,\
\"countdevices\" INTEGER,\
\"virtual\" INTEGER\
);\
INSERT INTO \"dsms\" (\"dsuid\",\"hwversion\",\"armswversion\",\"dspswversion\",\"apiversion\",\"name\",\"present\",\"apartmentid\",\"configid\",\"countdevices\",\"virtual\") \
SELECT \"dsuid\",\"hwversion\",\"armswversion\",\"dspswversion\",\"apiversion\",\"name\",\"present\",\"apartmentid\",\"configid\",\"countdevices\",\"virtual\" FROM \"tmp_dsms\";\
DROP TABLE \"tmp_dsms\";\
\
DROP TRIGGER IF EXISTS \"dsm_zonedata_cleanup\";\
\
DROP TRIGGER IF EXISTS \"dsm_cleanup\";\
CREATE TRIGGER \"dsm_cleanup\" \
BEFORE DELETE ON \"dsms\" \
FOR EACH ROW BEGIN \
DELETE FROM \"devices\" WHERE \"dsmdsuid\" = OLD.dsuid;\
DELETE FROM \"dsm_state\" WHERE \"dsuid\" = OLD.dsuid;\
END;\
\
CREATE TRIGGER \"dsm_update_device_count_on_insert\" \
AFTER INSERT ON \"devices\" \
BEGIN \
UPDATE \"dsms\" SET countdevices = (SELECT COUNT(*) FROM \"devices\" WHERE \"dsmdsuid\" = NEW.dsmdsuid) WHERE \"dsuid\" = NEW.dsmdsuid;\
END;\
\
CREATE TRIGGER \"dsm_update_device_count_on_delete\" \
AFTER DELETE ON \"devices\" \
FOR EACH ROW BEGIN \
UPDATE \"dsms\" SET countdevices = (SELECT COUNT(*) FROM \"devices\" WHERE \"dsmdsuid\" = OLD.dsmdsuid) WHERE \"dsuid\" = OLD.dsmdsuid;\
END;\
\
DROP TABLE IF EXISTS \"dsm_zones\";\
UPDATE \"dsa_config\" SET value=\'11\' WHERE key=\'db_version\';\
COMMIT;\
"

#define CREATE_DB_V12 "\
BEGIN TRANSACTION;\
ALTER TABLE \"devices\" ADD \"lastknowntemperature\" INTEGER DEFAULT -2732;\
UPDATE \"devices\" SET lastknowntemperature=\'-2732\';\
UPDATE \"dsa_config\" SET value=\'12\' WHERE key=\'db_version\';\
COMMIT;\
"

#define CREATE_DB_V13 "\
BEGIN TRANSACTION;\
CREATE TABLE \"taskdata_area_configuration\" (\
\"dsuid\" CHAR(35),\
\"zone\" INTEGER,\
\"group\" INTEGER,\
\"area1\" INTEGER,\
\"area2\" INTEGER,\
\"area3\" INTEGER,\
\"area4\" INTEGER,\
\"timestamp\" DATE,\
\"task_id\" INTEGER,\
FOREIGN KEY(\"task_id\") REFERENCES tasks(task_id)\
);\
UPDATE \"dsa_config\" SET value=\'13\' WHERE key=\'db_version\';\
COMMIT;\
"

#define CREATE_DB_V14 "\
BEGIN TRANSACTION;\
ALTER TABLE \"devices\" ADD \"chargeversion\" INTEGER DEFAULT -1;\
UPDATE \"devices\" SET chargeversion=\'-1\';\
UPDATE \"dsa_config\" SET value=\'14\' WHERE key=\'db_version\';\
COMMIT;\
"

#define CREATE_DB_V15 "\
BEGIN TRANSACTION;\
ALTER TABLE \"dsms\" ADD \"bustype\" INTEGER DEFAULT 0;\
UPDATE \"dsa_config\" SET value=\'15\' WHERE key=\'db_version\';\
COMMIT;\
"
#include "hwstats.sql"

#endif//__CREATE_DATABASE_H__
