/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Michael Tross <mt@aizo.ag>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifdef HAVE_CONFIG_H
    #include "autoconfig.h"
#endif

#include <boost/format.hpp>
#include "exceptions.h"
#include "hwstats.h"
#include "storage.h"
#include "util.h"

namespace dSA
{

stats_field_value_t device_sdesc_v1[] = {
{ STAT_VERSION, 1, unsignedint },
{ STAT_LENGTH, 1, unsignedint },
{ STAT_DSC11_TEMPERATURE, 1, dsc11temp },
{ STAT_POLARITY, 1, unsignedint },
{ STAT_SCENE_CURRENT, 1, unsignedint },
{ STAT_SCENE_PREVIOUS, 1, unsignedint },
{ STAT_CNT_OVERTEMP, 2, unsignedint },
{ STAT_CNT_OVERCURRENT, 2, unsignedint },
{ STAT_CNT_OVERLOAD, 2, unsignedint },
{ STAT_CNT_OVERLOAD_NET, 2, unsignedint },
{ STAT_TX_LONG_MSG, 2, unsignedint },
{ STAT_TX_SHORT_MSG, 2, unsignedint },
{ STAT_CNT_HALFWAVES, 4, zeroxcount },
{ STAT_RX_LONG_STARTWORD, 2, unsignedint },
{ STAT_RX_SHORT_STARTWORD, 2, unsignedint },
{ STAT_RX_LONG_MSG, 2, unsignedint },
{ STAT_RX_SHORT_MSG, 2, unsignedint },
{ STAT_RX_ERROR_CORRECTION_10, 2, unsignedint },
{ STAT_RX_ERROR_CORRECTION_01, 2, unsignedint },
{ STAT_RX_ERROR_CORRECTION_11, 2, unsignedint },
{ STAT_BITLEN_AVG_01, 2, unsignedint },
{ STAT_BITLEN_AVG_10, 2, unsignedint },
{ STAT_BITLEN_AVG_11, 2, unsignedint },
{ STAT_BITLEN_VAR_01, 4, unsignedint },
{ STAT_BITLEN_VAR_10, 4, unsignedint },
{ STAT_BITLEN_VAR_11, 4, unsignedint },
{ STAT_RX_TIMEOUT_BEG, 2, unsignedint },
{ STAT_RX_TIMEOUT_END, 2, unsignedint },
{ STAT_RX_ERROR_BEG, 2, unsignedint },
{ STAT_RX_ERROR_END, 2, unsignedint },
{ STAT_CNT_PULSE_NOFRAME_BEG, 2, unsignedint },
{ STAT_CNT_PULSE_NOFRAME_END, 2, unsignedint },
{ STAT_CNT_PULSE_FRAME_BEG, 2, unsignedint },
{ STAT_CNT_PULSE_FRAME_END, 2, unsignedint },
{ STAT_INVALID, 0, invalid }
};

stats_field_value_t device_sdesc_v4[] = {
{ STAT_VERSION, 1, unsignedint },
{ STAT_LENGTH, 1, unsignedint },
{ STAT_DSC11_TEMPERATURE, 1, dsc11temp },
{ STAT_POLARITY, 1, unsignedint },
{ STAT_SCENE_CURRENT, 1, unsignedint },
{ STAT_SCENE_PREVIOUS, 1, unsignedint },
{ STAT_CNT_OVERTEMP, 2, unsignedint },
{ STAT_CNT_OVERCURRENT, 2, unsignedint },
{ STAT_CNT_OVERLOAD, 2, unsignedint },
{ STAT_CNT_OVERLOAD_NET, 2, unsignedint },
{ STAT_TX_LONG_MSG, 2, unsignedint },
{ STAT_TX_SHORT_MSG, 2, unsignedint },
{ STAT_TX_SHORT_MSG_RESEND, 2, unsignedint },
{ STAT_CNT_HALFWAVES, 4, zeroxcount },
{ STAT_RX_STARTWORD, 2, unsignedint },
{ STAT_RX_LONG_MSG, 2, unsignedint },
{ STAT_RX_SHORT_MSG, 2, unsignedint },
{ STAT_RX_ERROR_CORRECTION_10, 2, unsignedint },
{ STAT_RX_ERROR_CORRECTION_01, 2, unsignedint },
{ STAT_RX_ERROR_CORRECTION_11, 2, unsignedint },
{ STAT_BITLEN_AVG_BEG, 1, unsignedint },
{ STAT_BITLEN_AVG_END, 1, unsignedint },
{ STAT_BITLEN_MIN_BEG, 1, unsignedint },
{ STAT_BITLEN_MIN_END, 1, unsignedint },
{ STAT_BITLEN_MAX_BEG, 1, unsignedint },
{ STAT_BITLEN_MAX_END, 1, unsignedint },
{ STAT_BITCNT_LENGTH_0, 4, unsignedint },
{ STAT_BITCNT_LENGTH_1, 4, unsignedint },
{ STAT_BITCNT_LENGTH_2, 4, unsignedint },
{ STAT_BITCNT_LENGTH_3, 4, unsignedint },
{ STAT_BITCNT_LENGTH_4, 4, unsignedint },
{ STAT_BITCNT_LENGTH_5, 4, unsignedint },
{ STAT_BITCNT_LENGTH_6, 4, unsignedint },
{ STAT_BITCNT_LENGTH_7, 4, unsignedint },
{ STAT_RX_TIMEOUT_BEG, 2, unsignedint },
{ STAT_RX_TIMEOUT_END, 2, unsignedint },
{ STAT_RX_ERROR_BEG, 2, unsignedint },
{ STAT_RX_ERROR_END, 2, unsignedint },
{ STAT_CNT_PULSE_NOFRAME_BEG, 2, unsignedint },
{ STAT_CNT_PULSE_NOFRAME_END, 2, unsignedint },
{ STAT_CNT_PULSE_FRAME_BEG, 2, unsignedint },
{ STAT_CNT_PULSE_FRAME_END, 2, unsignedint },
{ STAT_CNT_DSC11_RESET, 2, unsignedint },
{ STAT_CNT_OUTPUT_CHANGES, 2, unsignedint },
{ STAT_CNT_OUTPUT_ACTIVE, 4, zeroxcount },
{ STAT_CNT_LT_GLITCH, 1, unsignedint },
{ STAT_CNT_LT_BUTTON, 1, unsignedint },
{ STAT_CNT_EXT_GLITCH, 1, unsignedint },
{ STAT_CNT_EXT_BUTTON, 1, unsignedint },
{ STAT_RX_DISCARDED_LONG, 2, unsignedint },
{ STAT_RX_DISCARDED_SHORT, 2, unsignedint },
{ STAT_INVALID, 0, invalid }
};

stats_field_value_t meter_sdesc_v7[] = {
{ STAT_VERSION, 2, unsignedint },
{ STAT_RX_SHORT_MSG, 2, unsignedint },
{ STAT_RX_LONG_MSG, 2, unsignedint },
{ STAT_RX_COUNT_POSITIVE_HW, 2, unsignedint },
{ STAT_RX_ERROR_POSITIVE_HW, 2, unsignedint },
{ STAT_RX_ERROR_CRC_SHORT, 2, unsignedint },
{ STAT_RX_ERROR_CRC_LONG, 2, unsignedint },
{ STAT_RX_ERROR_HEADER_SHORT, 2, unsignedint },
{ STAT_RX_ERROR_HEADER_LONG, 2, unsignedint },
{ STAT_RX_QUALITY_AVG, 2, rxquality },
{ STAT_POWER_SILENCE_ONE, 4, unsignedint },
{ STAT_POWER_SILENCE_ZERO, 4, unsignedint },
{ STAT_POWER_SIGNAL_ONE, 4, unsignedint },
{ STAT_POWER_SIGNAL_ZERO, 4, unsignedint },
{ STAT_CNT_CARRIER_POWER, 4, unsignedint },
{ STAT_CNT_CARRIER_DETECT, 4, unsignedint },
{ STAT_CNT_CARRIER_REJECT, 4, unsignedint },
{ STAT_CNT_HALFWAVES, 4, zeroxcount },
{ STAT_SYNCPOS_POS_0, 2, unsignedint },
{ STAT_SYNCPOS_POS_1, 2, unsignedint },
{ STAT_SYNCPOS_POS_2, 2, unsignedint },
{ STAT_SYNCPOS_POS_3, 2, unsignedint },
{ STAT_SYNCPOS_POS_4, 2, unsignedint },
{ STAT_SYNCPOS_NEG_0, 2, unsignedint },
{ STAT_SYNCPOS_NEG_1, 2, unsignedint },
{ STAT_SYNCPOS_NEG_2, 2, unsignedint },
{ STAT_SYNCPOS_NEG_3, 2, unsignedint },
{ STAT_SYNCPOS_NEG_4, 2, unsignedint },
{ STAT_RX_QUALITY_AVG_ZERO_POS_0, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_1, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_2, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_3, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_4, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_5, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_6, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_7, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_8, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_0, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_1, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_2, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_3, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_4, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_5, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_6, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_7, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_8, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_0, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_1, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_2, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_3, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_4, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_5, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_6, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_7, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_8, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_0, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_1, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_2, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_3, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_4, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_5, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_6, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_7, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_8, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS, 2, carrierquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG, 2, carrierquality },
{ STAT_RX_QUALITY_AVG_ONE_POS, 2, carrierquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG, 2, carrierquality },
{ STAT_RX_BLANKED_AVG, 2, unsignedint },
{ STAT_RX_BLANKED_COUNT, 2, unsignedint },
{ STAT_CNT_EXCESS_DECODES, 2, unsignedint },
{ STAT_EQUALIZER_ONE, 1, unsignedint },
{ STAT_EQUALIZER_ZERO, 1, unsignedint },
{ STAT_HWMISMATCH_AVG, 1, signedint },
{ STAT_INVALID, 0, invalid }
};

stats_field_value_t meter_sdesc_v8[] = {
{ STAT_VERSION, 2, unsignedint },
{ STAT_RX_SHORT_MSG, 2, unsignedint },
{ STAT_RX_LONG_MSG, 2, unsignedint },
{ STAT_RX_COUNT_POSITIVE_HW, 2, unsignedint },
{ STAT_RX_ERROR_POSITIVE_HW, 2, unsignedint },
{ STAT_RX_ERROR_CRC_SHORT, 2, unsignedint },
{ STAT_RX_ERROR_CRC_LONG, 2, unsignedint },
{ STAT_RX_ERROR_HEADER_SHORT, 2, unsignedint },
{ STAT_RX_ERROR_HEADER_LONG, 2, unsignedint },
{ STAT_RX_QUALITY_AVG, 2, rxquality },
{ STAT_POWER_SILENCE_ONE, 4, unsignedint },
{ STAT_POWER_SILENCE_ZERO, 4, unsignedint },
{ STAT_POWER_SIGNAL_ONE, 4, unsignedint },
{ STAT_POWER_SIGNAL_ZERO, 4, unsignedint },
{ STAT_CNT_CARRIER_POWER, 4, unsignedint },
{ STAT_CNT_CARRIER_DETECT, 4, unsignedint },
{ STAT_CNT_CARRIER_REJECT, 4, unsignedint },
{ STAT_CNT_HALFWAVES, 4, zeroxcount },
{ STAT_SYNCPOS_POS_0, 2, unsignedint },
{ STAT_SYNCPOS_POS_1, 2, unsignedint },
{ STAT_SYNCPOS_POS_2, 2, unsignedint },
{ STAT_SYNCPOS_POS_3, 2, unsignedint },
{ STAT_SYNCPOS_POS_4, 2, unsignedint },
{ STAT_SYNCPOS_NEG_0, 2, unsignedint },
{ STAT_SYNCPOS_NEG_1, 2, unsignedint },
{ STAT_SYNCPOS_NEG_2, 2, unsignedint },
{ STAT_SYNCPOS_NEG_3, 2, unsignedint },
{ STAT_SYNCPOS_NEG_4, 2, unsignedint },
{ STAT_RX_QUALITY_AVG_ZERO_POS_0, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_1, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_2, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_3, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_4, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_5, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_6, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_7, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_8, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_0, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_1, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_2, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_3, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_4, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_5, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_6, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_7, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_8, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_0, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_1, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_2, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_3, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_4, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_5, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_6, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_7, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_8, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_0, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_1, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_2, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_3, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_4, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_5, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_6, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_7, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_8, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS, 2, carrierquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG, 2, carrierquality },
{ STAT_RX_QUALITY_AVG_ONE_POS, 2, carrierquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG, 2, carrierquality },
{ STAT_RX_BLANKED_AVG, 2, unsignedint },
{ STAT_RX_BLANKED_COUNT, 2, unsignedint },
{ STAT_CNT_EXCESS_DECODES, 2, unsignedint },
{ STAT_EQUALIZER_ONE, 1, unsignedint },
{ STAT_EQUALIZER_ZERO, 1, unsignedint },
{ STAT_HWMISMATCH_AVG, 1, signedint },
{ STAT_RESERVED, 1, unsignedint },
{ STAT_RESERVED, 1, unsignedint },
{ STAT_RESERVED, 1, unsignedint },
{ STAT_ADC_SATURATION_COUNT, 4, unsignedint },
{ STAT_INVALID, 0, invalid }
};

stats_field_value_t meter_sdesc_v9[] = {
{ STAT_VERSION, 2, unsignedint },
{ STAT_RX_SHORT_MSG, 2, unsignedint },
{ STAT_RX_LONG_MSG, 2, unsignedint },
{ STAT_RX_COUNT_POSITIVE_HW, 2, unsignedint },
{ STAT_RX_ERROR_POSITIVE_HW, 2, unsignedint },
{ STAT_RX_ERROR_CRC_SHORT, 2, unsignedint },
{ STAT_RX_ERROR_CRC_LONG, 2, unsignedint },
{ STAT_RX_ERROR_HEADER_SHORT, 2, unsignedint },
{ STAT_RX_ERROR_HEADER_LONG, 2, unsignedint },
{ STAT_RX_QUALITY_AVG, 2, rxquality },
{ STAT_POWER_SILENCE_ONE, 4, unsignedint },
{ STAT_POWER_SILENCE_ZERO, 4, unsignedint },
{ STAT_POWER_SIGNAL_ONE, 4, unsignedint },
{ STAT_POWER_SIGNAL_ZERO, 4, unsignedint },
{ STAT_CNT_CARRIER_POWER, 4, unsignedint },
{ STAT_CNT_CARRIER_DETECT, 4, unsignedint },
{ STAT_CNT_HALFWAVES, 4, zeroxcount },
{ STAT_SYNCPOS_POS_0, 2, unsignedint },
{ STAT_SYNCPOS_POS_1, 2, unsignedint },
{ STAT_SYNCPOS_POS_2, 2, unsignedint },
{ STAT_SYNCPOS_POS_3, 2, unsignedint },
{ STAT_SYNCPOS_POS_4, 2, unsignedint },
{ STAT_SYNCPOS_NEG_0, 2, unsignedint },
{ STAT_SYNCPOS_NEG_1, 2, unsignedint },
{ STAT_SYNCPOS_NEG_2, 2, unsignedint },
{ STAT_SYNCPOS_NEG_3, 2, unsignedint },
{ STAT_SYNCPOS_NEG_4, 2, unsignedint },
{ STAT_RX_QUALITY_AVG_ZERO_POS_0, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_1, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_2, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_3, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_4, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_5, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_6, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_7, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS_8, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_0, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_1, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_2, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_3, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_4, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_5, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_6, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_7, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG_8, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_0, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_1, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_2, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_3, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_4, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_5, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_6, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_7, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_POS_8, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_0, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_1, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_2, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_3, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_4, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_5, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_6, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_7, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG_8, 1, bitquality },
{ STAT_RX_QUALITY_AVG_ZERO_POS, 2, carrierquality },
{ STAT_RX_QUALITY_AVG_ZERO_NEG, 2, carrierquality },
{ STAT_RX_QUALITY_AVG_ONE_POS, 2, carrierquality },
{ STAT_RX_QUALITY_AVG_ONE_NEG, 2, carrierquality },
{ STAT_RX_BLANKED_AVG, 2, unsignedint },
{ STAT_RX_BLANKED_COUNT, 2, unsignedint },
{ STAT_CNT_EXCESS_DECODES, 2, unsignedint },
{ STAT_EQUALIZER_ONE, 1, unsignedint },
{ STAT_EQUALIZER_ZERO, 1, unsignedint },
{ STAT_HWMISMATCH_AVG, 1, signedint },
{ STAT_ADAPTIVE_SYNC_OFFSET_POS, 1, signedint },
{ STAT_ADAPTIVE_SYNC_OFFSET_NEG, 1, signedint },
{ STAT_ADAPTIVE_SYNC_MAX_OFFSET_POS, 1, signedint },
{ STAT_ADAPTIVE_SYNC_MAX_OFFSET_NEG, 1, signedint },
{ STAT_ADAPTIVE_SYNC_MIN_OFFSET_POS, 1, signedint },
{ STAT_ADAPTIVE_SYNC_MIN_OFFSET_NEG, 1, signedint },
{ STAT_RESERVED, 1, unsignedint },
{ STAT_ADC_SATURATION_COUNT, 4, unsignedint },
{ STAT_INVALID, 0, invalid }
};

stats_field_value_t filter_sdesc_v1[] = {
{ STAT_VERSION, 2, unsignedint },
{ STAT_CPU_CURRENT, 1, unsignedint },
{ STAT_CPU_AVG, 1, unsignedint },
{ STAT_CPU_MAX, 1, unsignedint },
{ STAT_CNT_RESET_DSF25, 1, unsignedint },
{ STAT_ADC_MIN, 2, unsignedint },
{ STAT_ADC_POW, 4, unsignedint },
{ STAT_ADC_MAX, 2, unsignedint },
{ STAT_CNT_ADC_CLIP, 2, unsignedint },
{ STAT_DAC_MIN, 2, unsignedint },
{ STAT_DAC_POW, 4, unsignedint },
{ STAT_DAC_MAX, 2, unsignedint },
{ STAT_CNT_DAC_CLIP, 2, unsignedint },
{ STAT_XF_MAX, 2, unsignedint },
{ STAT_XF_POW, 4, unsignedint },
{ STAT_CNT_XF_CLIP, 2, unsignedint },
{ STAT_LOWBAND_MAX, 2, unsignedint },
{ STAT_LOWBAND_POW, 4, unsignedint },
{ STAT_CNT_LOWBAND_CLIP, 2, unsignedint },
{ STAT_MAX_COEFF_LMS_0, 2, unsignedint },
{ STAT_MAX_COEFF_LMS_1, 2, unsignedint },
{ STAT_MAX_COEFF_LMS_2, 2, unsignedint },
{ STAT_MAX_COEFF_LMS_3, 2, unsignedint },
{ STAT_MAX_COEFF_LMS_4, 2, unsignedint },
{ STAT_MAX_COEFF_LMS_5, 2, unsignedint },
{ STAT_MAX_COEFF_LMS_6, 2, unsignedint },
{ STAT_MAX_COEFF_LMS_7, 2, unsignedint },
{ STAT_CNT_SAMPLES_SKIPPED, 2, unsignedint },
{ STAT_UPTIME, 4, unsignedint },
{ STAT_INVALID, 0, invalid }
};

StatisticValue::StatisticValue(statistic_field_t field, stats_value_t type, time_t t0, long long int value) :
        m_field(field), m_type(type), m_time(t0), m_value(value), m_errorcode(0)
{
}

StatisticValue::StatisticValue(statistic_field_t field, stats_value_t type, time_t t0, int errorcode) :
        m_field(field), m_type(type), m_time(t0), m_value(0), m_errorcode(errorcode)
{
}

StatisticBase::StatisticBase(uint16_t version)
{
    m_version = version;
    m_index = 0;
    setDescriptionTable(NULL, 0);
    memset(m_data, 0, sizeof(m_data));
    memset(m_valid, 0, sizeof(m_valid));
}

StatisticBase::~StatisticBase() {}

uint16_t StatisticBase::getVersion() const
{
    return m_version;
}

uint8_t StatisticBase::getValue8(uint8_t index) const
{
    return m_data[index];
}

uint16_t StatisticBase::getValue16(uint8_t index) const
{
    uint16_t rval =
            (m_data[index+1] << 8) |
            (m_data[index]);
    return rval;
}

uint32_t StatisticBase::getValue32(uint8_t index) const
{
    uint32_t rval =
            (m_data[index+3] << 24) |
            (m_data[index+2] << 16) |
            (m_data[index+1] << 8) |
            (m_data[index+0]);
    return rval;
}

uint64_t StatisticBase::getValue64(uint8_t index) const
{
    uint64_t rval = ((uint64_t) getValue32(index+4) << 32) | (uint64_t) getValue32(index);
    return rval;
}

std::string StatisticBase::getValueString(const stats_field_value_t *item, int offset)
{
    std::string result;

    switch (item->type)
    {
    case unsignedint:
    case signedint:
    default:
        switch (item->size)
        {
        case 1:
        {
            uint8_t val = getValue8(offset);
            result = util::hex(val, 2);
        }
        break;
        case 2:
        {
            uint16_t val = getValue16(offset);
            result = util::hex(val, 4);
        }
        break;
        case 4:
        {
            uint32_t val = getValue32(offset);
            result = util::hex(val, 8);
        }
        break;
        default:
        {
            result = util::hex(m_data + offset, item->size);
        }
        break;
        }

        break;
    case zeroxcount:
    {
        uint32_t val = getValue32(offset);
        result = util::hex(val, 8);
    }
    break;

    case dsc11temp:
    {
        uint8_t val = getValue8(offset);
        result = util::hex(val, 2);
    }
    break;
    }

    return result;
}

long long int StatisticBase::getValueInt(const stats_field_value_t *item, int offset)
{
    long long int result;

    switch (item->type)
    {
    case signedint:
        switch (item->size)
        {
        case 1:
            result = static_cast<int8_t>(getValue8(offset));
            break;
        case 2:
            result = static_cast<int16_t>(getValue16(offset));
            break;
        case 4:
            result = static_cast<int32_t>(getValue32(offset));
            break;
        case 8:
            result = static_cast<int64_t>(getValue64(offset));
            break;
        default:
            result = -1;
            break;
        }
        break;
    case unsignedint:
    default:
        switch (item->size)
        {
        case 1:
            result = getValue8(offset);
            break;
        case 2:
            result = getValue16(offset);
            break;
        case 4:
            result = getValue32(offset);
            break;
        case 8:
            result = getValue64(offset);
            break;
        default:
            result = -1;
            break;
        }
        break;

    case zeroxcount:
        result = getValue32(offset);
        break;

    case dsc11temp:
        result = getValue8(offset);
        break;
    }

    return result;
}

void StatisticBase::pushValue(uint8_t val, int errorcode)
{
    if (!errorcode)
    {
        m_valid[m_index] = true;
        m_data[m_index] = val;
    }
    m_time[m_index] = time(NULL);
    m_error[m_index] = errorcode;
    m_index ++;
}

void StatisticBase::pushValue(uint16_t val, int errorcode)
{
    pushValue( (uint8_t) (val & 0xff), errorcode);
    pushValue( (uint8_t) (val >> 8), errorcode);
}

void StatisticBase::setValue(uint8_t offset, uint16_t val, int errorcode)
{
    uint8_t oindex = m_index;
    m_index = offset;
    pushValue( (uint8_t) (val & 0xff), errorcode);
    pushValue( (uint8_t) (val >> 8), errorcode);
    m_index = oindex;
}

bool StatisticBase::isValid(statistic_field_t index)
{
    stats_field_value_t* t = m_definition_table;
    int offset = 0;
    bool incomplete = false;

    while (t->field != index)
    {
        offset += t->size;
        t ++;
    }
    if (t->field != index)
    {
        throw _Exception("Statistic field " + util::str(index) + " is not valid");
    }
    for (int n = 0; n < t->size; n ++)
    {
        if (!m_valid[offset+n])
        {
            incomplete = true;
        }
    }
    return !incomplete;
}

void StatisticBase::parseValues()
{
    stats_field_value_t* t;
    int offset;
    int errorcode;

    m_value_map.reset();
    m_value_map = boost::shared_ptr<std::list<boost::shared_ptr<StatisticValue> > >
        (new std::list<boost::shared_ptr<StatisticValue> >);

    for (t = m_definition_table, offset = 0;
            t->field != STAT_INVALID;
            offset += t->size, t++)
    {
        errorcode = 0;
        for (int n = 0; n < t->size; n ++)
        {
            if (!m_valid[offset+n])
            {
                errorcode = -255;
            }
            if (m_error[offset+n])
            {
                errorcode = m_error[offset+n];
            }
        }
        if (errorcode)
        {
            m_value_map->push_back(boost::shared_ptr<StatisticValue>
                (new StatisticValue(t->field, t->type, m_time[offset], errorcode)));
        }
        else
        {
            m_value_map->push_back(boost::shared_ptr<StatisticValue>
                (new StatisticValue(t->field, t->type, m_time[offset], getValueInt(t, offset))));
        }
    }
}

void StatisticBase::getOffsetForType(statistic_field_t field, uint8_t &_offset, uint8_t &_length)
{
    stats_field_value_t* t;
    int offset;

    for (t = m_definition_table, offset = 0;
            t->field != STAT_INVALID;
            offset += t->size, t++)
    {
        if (t->field == field)
        {
            _offset = offset;
            _length = t->size;
            return;
        }
    }
    throw Exception("Invalid statistic field type");
}

boost::shared_ptr<StatisticValue> StatisticBase::getValueForType(statistic_field_t field)
{
    for (std::list<boost::shared_ptr<StatisticValue> >::iterator it = m_value_map->begin();
            it != m_value_map->end();
            it++)
    {
        if ((*it)->m_field == field)
            return (*it);
    }
    throw Exception("No value for statistic field type found");
}

std::string StatisticBase::formatValue(boost::shared_ptr<StatisticValue> pval)
{
    std::string result;

    if (pval->m_errorcode) {
        result = std::string("ERROR: ") + util::str(pval->m_errorcode);
        return result;
    }

    switch (pval->m_type)
    {
        case unsignedint:
        case signedint:
        default:
        {
            uint8_t o, l;
            getOffsetForType(pval->m_field, o, l);
            result = util::hex( (int) pval->m_value, l);
        }
        break;

        case rxquality:
        {
            uint16_t val = (uint16_t) pval->m_value;
            float vpercent = val * 100.0 / 15872.0;
            std::string r(boost::str(
                boost::format(
                    "%1%"
                    ) % vpercent ));
            result = util::str(val) + " (" + r + "%)";
        }
        break;

        case bitquality:
        {
            uint16_t val = (uint16_t) pval->m_value;
            float vpercent = (1 - val / 248.0) * 100.0;
            std::string r(boost::str(
                boost::format(
                    "%1%"
                    ) % vpercent ));
            result = util::str(val) + " (" + r + "%)";
        }
        break;

        case carrierquality:
        {
            uint16_t val = (uint16_t) pval->m_value;
            float vpercent = (1 - val / 63488.0) * 100.0;
            std::string r(boost::str(
                boost::format(
                    "%1%"
                    ) % vpercent ));
            result = util::str(val) + " (" + r + "%)";
        }
        break;

        case zeroxcount:
            {
                uint32_t val = (uint32_t) pval->m_value;
                uint32_t seconds = (val + 50) / 100;
                uint32_t minutes = seconds / 60; seconds %= 60;
                uint32_t hours = minutes / 60; minutes %= 60;
                uint32_t days = hours / 24; hours %= 24;
                std::string uptime(boost::str(
                    boost::format(
                        "%1%d %2%h %3%m %4%s"
                        ) % days % hours % minutes % seconds ));
                result = util::hex(val, 8) + " (" + uptime + ")";
            }
            break;

        case dsc11temp:
            {
                uint8_t val = (uint8_t) pval->m_value;
                std::string celsius(boost::str(
                    boost::format("%1% °C, +/-10") % (149 - 0.9 * val)));
                result = util::hex(val, 2) + " (" + celsius + ")";
            }
            break;
    }
    return result;
}

boost::shared_ptr<std::map<statistic_field_t,std::string> > StatisticBase::formatValues()
{
    boost::shared_ptr<std::map<statistic_field_t, std::string> > result(
                                  new std::map<statistic_field_t,std::string>);
    std::list<boost::shared_ptr<StatisticValue> >::iterator it;

    for (it = m_value_map->begin(); it != m_value_map->end(); it++)
    {
        result->insert( std::make_pair ((*it)->m_field, formatValue(*it)));
    }

    return result;
}

void StatisticBase::setDescriptionTable(const stats_field_value_t *table, int length)
{
    m_definition_table = (stats_field_value_t *) table;
    m_definition_table_length = length;
}

StatisticDevice::StatisticDevice(uint8_t version) : StatisticBase(version)
{
    // select defintion table based on version
    switch (version) {
        case 0x01:
        case 0x02:
            setDescriptionTable(device_sdesc_v1,
                    sizeof(device_sdesc_v1) / sizeof(stats_field_value_t) - 1);
            break;
        case 0x03:
        case 0x04:
        default:
            setDescriptionTable(device_sdesc_v4,
                    sizeof(device_sdesc_v4) / sizeof(stats_field_value_t) - 1);
            break;
    }
}

StatisticMeter::StatisticMeter(uint16_t version) : StatisticBase(version)
{
    // select defintion table based on version
    switch (version)
    {
        case 0x07:
            setDescriptionTable(meter_sdesc_v7,
                    sizeof(meter_sdesc_v7) / sizeof(stats_field_value_t) - 1);
            break;
        case 0x08:
            setDescriptionTable(meter_sdesc_v8,
                    sizeof(meter_sdesc_v8) / sizeof(stats_field_value_t) - 1);
            break;
        case 0x09:
        default:
            setDescriptionTable(meter_sdesc_v9,
                    sizeof(meter_sdesc_v9) / sizeof(stats_field_value_t) - 1);
            break;
    }
}

StatisticFilter::StatisticFilter(uint16_t version) : StatisticBase(version)
{
    // select defintion table based on version
    switch (version)
    {
        case 0x01:
        default:
            setDescriptionTable(filter_sdesc_v1,
                    sizeof(filter_sdesc_v1) / sizeof(stats_field_value_t) - 1);
            break;
    }
}

}
